from os.path import join
from PyQt6.QtCore import QFile
from PyQt6 import uic
from PyQt6.QtCore import QIODevice


def load_ui_from_file(name: str, dir_path: str):
    """
    Returns a tuple from uic.loadUiType(), loading the ui file with the given name.
    :param name:
    :return:
    """
    ui_file = _get_ui_qfile(name, dir_path)
    try:
        base_type = uic.loadUiType(ui_file)
    finally:
        ui_file.close()
    return base_type


def _get_ui_qfile(name: str, dir_path: str) -> QFile:
    """
    Returns an opened, read-only QFile for the given QtDesigner UI file name. Expects a plain name like "centralwidget".
    The file ending and resource path is added automatically.
    Raises FileNotFoundError, if the given ui file does not exist.
    :param name:
    :return:
    """
    file_path = join(dir_path,f"{name}.ui")
    file = QFile(file_path)
    if not file.exists():
        raise FileNotFoundError("UI file not found: " + file_path)
    file.open(QIODevice.OpenModeFlag.ReadOnly)
    return file


"""
This renamed function is supposed to be used during class definition to make the intention clear.
Usage example:
class SomeWidget(*inherits_from_ui_file_with_name("SomeWidgetUiFileName")):
    def __init__(self, parent):
        super(SomeWidget, self).__init__(parent)
        self.setupUi(self)
"""
inherits_from_ui_file_with_name = load_ui_from_file



from typing import Optional
from os.path import exists

from PyQt6.QtWidgets import QDialog, QWidget, QVBoxLayout, QTextBrowser, QDialogButtonBox
from PyQt6.QtCore import Qt
from markdown import markdown


class ChangeLogDialog(QDialog):

    def __init__(self, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent)

        layout = QVBoxLayout(self)

        self.setLayout(layout)

        self.setWindowTitle("Change Log")

        self.textBrowser = QTextBrowser(self)

        self.buttonBox = QDialogButtonBox(self)
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.addButton(QDialogButtonBox.StandardButton.Close)
        self.buttonBox.rejected.connect(self.reject)

        self.layout().addWidget(self.textBrowser)
        self.layout().addWidget(self.buttonBox)

    def setFile(self, filepath: str) -> None:
        """Loads converted MD file into text browser
        """

        if not exists(change_log := filepath):
            raise Exception(f"Unable to find change log file ({change_log})")

        html = ConvertMdToHtml(change_log)

        self.textBrowser.setText(html)

def ConvertMdToHtml(file_path: str) -> str:
        """Reads file and converts MD to Html

        Args:
            file_path (str): MD file path

        Returns:
            str: Converted Html
        """

        with open(file_path) as file:
            return markdown(file.read())


"""Adds helpfule tools for pyqt apps"""

from .inherits_ui_file import inherits_from_ui_file_with_name
from .change_log_dialog import ChangeLogDialog

__all__ = ["inherits_from_ui_file_with_name","ChangeLogDialog"]

__version__ = "0.1.13"

__module_name__ = "pyqtapplicationhelper"



from os import getcwd, walk
from os.path import join

from typing import List

from setuptools import setup
from pyqtapplicationhelper import __version__, __module_name__

def get_dependancies() -> List[str]:
    """Gets dependancies from requirements.txt"""
    dependancies = []

    requirements_file = join(getcwd(), "requirements.txt")

    with open(requirements_file,"r", encoding="UTF-8") as file:
        dependancies = file.read().split("\n")
        dependancies = filter(None, dependancies)
        dependancies = filter(bool, dependancies)
        dependancies = filter(len, dependancies)
        dependancies = list(filter(None, dependancies))

    return dependancies

def get_packages() -> List[str]:
    """Gets all subpackages"""
    directory = join(".","pyqtapplicationhelper")

    _packages = [x[0].replace("\\",".") for x in walk(directory) if "__pycache__" not in x[0]]

    return _packages

setup(
    name = __module_name__,
    version = __version__,
    author = "michaeldcanady",
    author_email = "",
    description = (""),
    license = "MIT",
    #keywords = "example documentation tutorial",
    url = "https://github.com/michaeldcanady/pyqtapplicationhelper",
    packages=get_packages(),
    install_requires=get_dependancies()
    #long_description=read('README'),
)